<?php

declare(strict_types=1);

namespace Dedi\SyliusCustomerPoolPlugin\Provider;

use Dedi\SyliusCustomerPoolPlugin\Model\Customer\CustomerPoolAwareInterface;
use Dedi\SyliusCustomerPoolPlugin\Repository\UserWithCustomerPoolRepositoryInterface;
use Sylius\Bundle\UserBundle\Provider\AbstractUserProvider;
use Sylius\Component\Channel\Context\ChannelContextInterface;
use Symfony\Component\Security\Core\User\UserInterface;

class EmailAndCustomerPoolProvider extends AbstractUserProvider
{
    private ChannelContextInterface $channelContext;

    public function setChannelContext(ChannelContextInterface $channelContext): void
    {
        $this->channelContext = $channelContext;
    }

    protected function findUser(string $uniqueIdentifier): ?UserInterface
    {
        /** @var ChannelContextInterface|CustomerPoolAwareInterface $channel */
        $channel = $this->channelContext->getChannel();

        if (null === ($channelCustomerPool = $channel->getCustomerPool())) {
            return $this->userRepository->findOneByEmail($uniqueIdentifier);
        }

        if (!$this->userRepository instanceof UserWithCustomerPoolRepositoryInterface) {
            throw new \InvalidArgumentException(sprintf(
                'Expected an instance of %s. Got: %s',
                UserWithCustomerPoolRepositoryInterface::class,
                get_class($this->userRepository)
            ));
        }

        /** @var UserWithCustomerPoolRepositoryInterface $repository */
        $repository = $this->userRepository;

        return $repository->findOneByEmailAndCustomerPool($uniqueIdentifier, $channelCustomerPool);
    }
}
